#ifndef ABCDISK_H
#define ABCDISK_H

#include "compiler.h"
#include "patchlevel.h"

/*
 * Windows has a different signature for mkdir(), but fortunately also
 * has an alias _mkdir() to look for...
 */
#ifdef HAVE__MKDIR
# define make_dir(x) _mkdir(x)
#else
# define make_dir(x) mkdir((x), 0777)
#endif

struct diskfmt {
    const char *name;
    unsigned char c, h, s;
    int sectors;                /* Total sectors (max 1911 clusters) */
    unsigned int cluster_shift; /* Size of a cluster, log2 */
    int syssec;                 /* First system sector (bitmap) */
    int dirsec[2];              /* First directory sector (main, backup) */
    int dirsecs;                /* Number of directory sectors */
};

struct disk;

struct directory {
    struct disk *disk;          /* This directory belongs to... */
    unsigned char *bitmap;      /* Directory entry "bitmap" area */
    unsigned char *dirsec;      /* Actual directory sector */
    unsigned int block;         /* UFD block pointer (0 for MFD) */
};

struct disk {
    const struct diskfmt *fmt;  /* Format descriptor */
    unsigned char *image;       /* Image of the disk */
    unsigned char *bitmap;      /* Pointer to the bitmap inside the disk */
    struct directory mfd;       /* MFD directory descriptor */
};

const struct diskfmt *get_format(const char *);
void show_formats(FILE *);
struct disk * safe_alloc allocate_disk(const struct diskfmt *fmt);

extern const char *program;
void * safe_malloc(1) xmalloc(size_t);
void * safe_malloc(1) xzalloc(size_t);

static inline void mark_used(struct disk *disk, unsigned int cluster)
{
    disk->bitmap[cluster >> 3] |= (0x80 >> (cluster & 7));
}

static inline int is_used(struct disk *disk, unsigned int cluster)
{
    return !!(disk->bitmap[cluster >> 3] & (0x80 >> (cluster & 7)));
}

/*
 * Convert a filename in ABC format to a lower-case filename
 * in host encoding
 */
void unmangle_filename(char *out, const unsigned char *in);

/*
 * Convert a filename in host encoding to a filename in ABC format
 * Returns 0 for OK, -1 for failure (filename cannot be converted.)
 */
int mangle_name(unsigned char *dst, const char *src);

#endif /* ABCDISK_H */
